# -*- coding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2017-TODAY Eynes (<http://www.e-mips.com.ar>)
#
#    This is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp import _, api, models, fields
from openerp.addons import decimal_precision as dp


class FinancingConfig(models.Model):
    _name = 'financing.config'
    _description = 'Financing General Configuration'
    _order = "customer_type desc, quota_qty"
    _rec_name = "quota_qty"

    @api.depends("quota_qty", "monthly_rate")
    def compute_total_rate(self):
        for pc in self:
            try:
                pc.total_rate = int(pc.quota_qty) * pc.monthly_rate
            except ValueError:
                pc.total_rate = 0

    quota_qty = fields.Integer(string='Quota Qty.', required=True, default=0)
    monthly_rate = fields.Float(string='Monthly Rate', required=True, default=0,
                                digits=dp.get_precision('Financing Config.'))
    total_rate = fields.Float(string='Total Rate', compute="compute_total_rate",
                              digits=dp.get_precision('Financing Config.'))
    customer_type = fields.Selection(
        [
            ("associated", "Associated"),
            ("non_associated", "Non-associated"),
        ],
        string="Customer type",
        default="non_associated",
        required=True,
    )

    _sql_constraints = [
        (
            "check_quota_qty_positive",
            "CHECK(quota_qty > 0)",
            _("Quota Qty. must be > 0!")
        ),
        (
            "check_monthly_rate_positive",
            "CHECK(NOT monthly_rate < 0)",
            _("Monthly Rate must be >= 0!")
        ),
        (
            "uniq_quota_qty_product",
            "UNIQUE(quota_qty, customer_type)",
            _("You already configured this financing for this Quota")
        ),
    ]


class ProductFinancing(models.Model):
    _name = 'product.financing'
    _description = 'Product Financing data'
    _order = "product_id, pos_config_id, customer_type desc, financing_config_id"

    @api.onchange("customer_type")
    def onchange_customer_type(self):
        if self.financing_config_id and self.customer_type:
            new_config = self.env['financing.config'].search(
                [
                    ("quota_qty", "=", self.financing_config_id.quota_qty),
                    ("customer_type", "=", self.customer_type),
                ],
            )
            self.financing_config_id = new_config.id

    @api.depends("quota_amount", "financing_config_id")
    def compute_total(self):
        for pf in self:
            pf.total = pf.quota_amount * pf.quota_qty
            pf.quota_amount_taxed = pf.quota_amount * 1.21
            pf.total_taxed = pf.quota_amount * pf.quota_qty * 1.21

    @api.onchange("financing_config_id", "customer_type", "product_id")
    def onchange_financing(self):
        config = self.financing_config_id
        if not self.product_id or not config:
            self.quota_amount = 0
            return

        lst_price = self.product_id.lst_price
        quota_qty = config.quota_qty
        total_rate = config.total_rate or 0
        self.quota_amount = (lst_price * (1 + total_rate)) / quota_qty

    #@api.onchange("customer_type")
    #def change_customer_type(self):
    #    query = """
    #    SELECT quota_qty, quota_amount
    #    FROM product.financing
    #    WHERE product_id = %s
    #        AND customer_type = %s
    #    ORDER BY quota_qty desc
    #    LIMIT 1
    #    """
    #    product_id = self.env.context.get("active_id", False)
    #    if self.customer_type and product_id:
    #        self.env.cr.execute(query, (product_id, self.customer_type))
    #        try:
    #            last_quota_qty, last_quota_amount = self.enc.cr.fetchone()
    #            q, r = divmod(last_quota_amount, 3)
    #            self.quota_qty = r == 0 and (q + 1) * 3 or last_quota_amount + 1
    #            self.quota_amount = last_quota_amount
    #        except (TypeError, IndexError):
    #            self.quota_qty = 1
    #            self.quota_amount = 0

    def default_pos(self):
        return self.env["pos.config"].search([], order="id", limit=1)

    product_id = fields.Many2one('product.product', string='Product', required=True,
                                 ondelete='cascade')
    financing_config_id = fields.Many2one('financing.config', string='Quota Qty.', required=True)
    pos_config_id = fields.Many2one('pos.config', string='PoS', required=True, default=default_pos)
    quota_qty = fields.Integer(related="financing_config_id.quota_qty")
    quota_amount = fields.Float(string='Quota Amount',
                                digits=dp.get_precision('Financing Config.'))
    total = fields.Float(string='Total', compute="compute_total",
                         digits=dp.get_precision('Financing Config.'))
    quota_amount_taxed = fields.Float(string='Quota Amount Taxed', compute="compute_total",
                                      digits=dp.get_precision('Financing Config.'))
    total_taxed = fields.Float(string='Total Taxed', compute="compute_total",
                               digits=dp.get_precision('Financing Config.'))
    tfc = fields.Float(string='T.F.C.', digits=dp.get_precision('Financing Config.'))
    apr = fields.Float(string='A.P.R.', digits=dp.get_precision('Financing Config.'))
    customer_type = fields.Selection(
        [
            ("associated", "Associated"),
            ("non_associated", "Non-associated"),
        ],
        string="Customer type",
        default="non_associated",
        required=True,
    )

    _sql_constraints = [
        (
            "uniq_quota_qty_product",
            "UNIQUE(product_id, financing_config_id, customer_type, pos_config_id)",
            _("You already configured this financing for this product")
        ),
    ]
